import os

from zrb.group.any_group import AnyGroup
from zrb.util.file import read_file
from zrb.util.group import get_non_empty_subgroups, get_subtasks
from zrb.util.string.format import fstring_format

_DIR = os.path.dirname(__file__)

_VIEW_TEMPLATE = read_file(os.path.join(_DIR, "view.html"))
_GROUP_INFO_TEMPLATE = read_file(os.path.join(_DIR, "partial", "group_info.html"))
_GROUP_LI_TEMPLATE = read_file(os.path.join(_DIR, "partial", "group_li.html"))
_TASK_INFO_TEMPLATE = read_file(os.path.join(_DIR, "partial", "task_info.html"))
_TASK_LI_TEMPLATE = read_file(os.path.join(_DIR, "partial", "task_li.html"))


def handle_home_page(root_group: AnyGroup):
    from fastapi.responses import HTMLResponse

    subgroups = get_non_empty_subgroups(root_group, web_only=True)
    group_info = (
        ""
        if len(subgroups) == 0
        else fstring_format(
            _GROUP_INFO_TEMPLATE,
            {
                "group_li": "\n".join(
                    [
                        fstring_format(
                            _GROUP_LI_TEMPLATE,
                            {"caption": name, "description": group.description},
                        )
                        for name, group in subgroups.items()
                    ]
                )
            },
        )
    )
    subtasks = get_subtasks(root_group, web_only=True)
    task_info = (
        ""
        if len(subtasks) == 0
        else fstring_format(
            _TASK_INFO_TEMPLATE,
            {
                "task_li": "\n".join(
                    [
                        fstring_format(
                            _TASK_LI_TEMPLATE,
                            {"caption": name, "description": task.description},
                        )
                        for name, task in subtasks.items()
                    ]
                )
            },
        )
    )
    return HTMLResponse(
        fstring_format(
            _VIEW_TEMPLATE,
            {
                "group_info": group_info,
                "task_info": task_info,
                "name": root_group.name,
                "description": root_group.description,
            },
        )
    )



import os

from my_app_name._zrb.config import APP_DIR
from my_app_name._zrb.util import get_existing_module_names

from zrb.context.any_context import AnyContext
from zrb.util.codemod.append_key_to_dict import append_key_to_dict
from zrb.util.file import read_file, write_file
from zrb.util.string.conversion import to_kebab_case, to_pascal_case, to_snake_case


def is_app_config_file(ctx: AnyContext, file_path: str) -> bool:
    return file_path == os.path.join(APP_DIR, "config.py")


def is_app_main_file(ctx: AnyContext, file_path: str) -> bool:
    return file_path == os.path.join(APP_DIR, "main.py")


def is_app_zrb_task_file(ctx: AnyContext, file_path: str) -> bool:
    return file_path == os.path.join(APP_DIR, "_zrb", "task.py")


def is_app_zrb_config_file(ctx: AnyContext, file_path: str) -> bool:
    return file_path == os.path.join(APP_DIR, "_zrb", "config.py")


def is_in_module_dir(ctx: AnyContext, file_path: str) -> bool:
    return file_path.startswith(
        os.path.join(APP_DIR, "module", to_snake_case(ctx.input.module))
    )


def update_app_zrb_config_file(ctx: AnyContext, zrb_config_file_path: str):
    existing_zrb_config_code = read_file(zrb_config_file_path)
    module_name = ctx.input.module
    upper_snake_module_name = to_snake_case(module_name).upper()
    existing_module_names = get_existing_module_names()
    module_port = 3001 + len(
        [
            existing_module_name
            for existing_module_name in existing_module_names
            if existing_module_name != to_snake_case(module_name)
        ]
    )
    write_file(
        file_path=zrb_config_file_path,
        content=append_key_to_dict(
            original_code=existing_zrb_config_code,
            dictionary_name="MICROSERVICES_ENV_VARS",
            new_key=f"MY_APP_NAME_{upper_snake_module_name}_BASE_URL",
            new_value=f"http://localhost:{module_port}",
        ),
    )


def update_app_zrb_task_file(ctx: AnyContext, zrb_task_file_path: str):
    existing_zrb_task_code = read_file(zrb_task_file_path)
    write_file(
        file_path=zrb_task_file_path,
        content=[
            existing_zrb_task_code.strip(),
            "",
            _get_task_definition_code(existing_zrb_task_code, ctx.input.module),
        ],
    )


def _get_task_definition_code(existing_code: str, module_name: str) -> str | None:
    existing_module_names = get_existing_module_names()
    module_port = 3001 + len(
        [
            existing_module_name
            for existing_module_name in existing_module_names
            if existing_module_name != to_snake_case(module_name)
        ]
    )
    snake_module_name = to_snake_case(module_name)
    kebab_module_name = to_kebab_case(module_name)
    pascal_module_name = to_pascal_case(module_name)
    task_definition_code = read_file(
        file_path=os.path.join(
            os.path.dirname(__file__), "template", "module_task_definition.py"
        ),
        replace_map={
            "my_module": snake_module_name,
            "my-module": kebab_module_name,
            "My Module": pascal_module_name,
            "3000": f"{module_port}",
        },
    ).strip()
    if task_definition_code in existing_code:
        return None
    return task_definition_code


def update_app_main_file(ctx: AnyContext, app_main_file_path: str):
    existing_app_main_code = read_file(app_main_file_path)
    write_file(
        file_path=app_main_file_path,
        content=[
            _get_import_module_route_code(existing_app_main_code, ctx.input.module),
            existing_app_main_code,
            "",
            _get_assert_module_route_code(existing_app_main_code, ctx.input.module),
        ],
    )


def _get_import_module_route_code(existing_code: str, module_name: str) -> str | None:
    snake_module_name = to_snake_case(module_name)
    import_module_path = f"my_app_name.module.{snake_module_name}"
    import_route_code = (
        f"from {import_module_path} import route as {snake_module_name}_route"
    )
    if import_route_code in existing_code:
        return None
    return import_route_code


def _get_assert_module_route_code(existing_code: str, module_name: str) -> str | None:
    snake_module_name = to_snake_case(module_name)
    assert_route_code = f"assert {snake_module_name}_route"
    return assert_route_code if assert_route_code not in existing_code else None


def update_app_config_file(ctx: AnyContext, module_config_path: str):
    existing_config_code = read_file(module_config_path)
    write_file(
        module_config_path,
        [
            existing_config_code.strip(),
            _get_new_module_config_code(existing_config_code, ctx.input.module),
        ],
    )


def _get_new_module_config_code(existing_code: str, module_name: str) -> str | None:
    existing_module_names = get_existing_module_names()
    module_port = 3000 + len(
        [
            existing_module_name
            for existing_module_name in existing_module_names
            if existing_module_name != to_snake_case(module_name)
        ]
    )
    module_base_url = f"http://localhost:{module_port}"
    upper_snake_module_name = to_snake_case(module_name).upper()
    config_name = f"APP_{upper_snake_module_name}_BASE_URL"
    env_name = f"MY_APP_NAME_{upper_snake_module_name}_BASE_URL"
    config_code = f'{config_name} = os.getenv("{env_name}", "{module_base_url}")'
    if config_code in existing_code:
        return None
    return config_code



import os

from my_app_name._zrb.config import APP_DIR

from zrb.context.any_context import AnyContext
from zrb.util.codemod.append_code_to_class import append_code_to_class
from zrb.util.codemod.append_code_to_function import append_code_to_function
from zrb.util.codemod.prepend_code_to_module import prepend_code_to_module
from zrb.util.codemod.prepend_parent_to_class import prepend_parent_class
from zrb.util.file import read_file, write_file
from zrb.util.string.conversion import to_pascal_case, to_snake_case


def is_in_app_schema_dir(ctx: AnyContext, file_path: str) -> bool:
    return file_path.startswith(
        os.path.join(APP_DIR, "schema", to_snake_case(ctx.input.entity))
    )


def is_in_module_entity_dir(ctx: AnyContext, file_path: str) -> bool:
    return file_path.startswith(
        os.path.join(
            APP_DIR,
            "module",
            to_snake_case(ctx.input.module),
            "service",
            to_snake_case(ctx.input.entity),
        )
    )


def is_module_route_file(ctx: AnyContext, file_path: str) -> bool:
    module_route_file = os.path.join(
        APP_DIR,
        "module",
        to_snake_case(ctx.input.module),
        "route.py",
    )
    return file_path == module_route_file


def is_module_migration_metadata_file(ctx: AnyContext, file_path: str) -> bool:
    module_migration_metadata_file = os.path.join(
        APP_DIR,
        "module",
        to_snake_case(ctx.input.module),
        "migration_metadata.py",
    )
    return file_path == module_migration_metadata_file


def is_module_any_client_file(ctx: AnyContext, file_path: str) -> bool:
    module_any_client_file = os.path.join(
        APP_DIR,
        "module",
        to_snake_case(ctx.input.module),
        "client",
        "any_client.py",
    )
    return file_path == module_any_client_file


def is_module_api_client_file(ctx: AnyContext, file_path: str) -> bool:
    module_api_client_file = os.path.join(
        APP_DIR,
        "module",
        to_snake_case(ctx.input.module),
        "client",
        "api_client.py",
    )
    return file_path == module_api_client_file


def is_module_direct_client_file(ctx: AnyContext, file_path: str) -> bool:
    module_direct_client_file = os.path.join(
        APP_DIR,
        "module",
        to_snake_case(ctx.input.module),
        "client",
        "direct_client.py",
    )
    return file_path == module_direct_client_file


def update_migration_metadata(ctx: AnyContext, migration_metadata_file_path: str):
    app_name = os.path.basename(APP_DIR)
    existing_migration_metadata_code = read_file(migration_metadata_file_path)
    write_file(
        file_path=migration_metadata_file_path,
        content=[
            _get_import_schema_code(
                existing_migration_metadata_code, app_name, ctx.input.entity
            ),
            existing_migration_metadata_code.strip(),
            _get_entity_metadata_assignment_code(
                existing_migration_metadata_code, ctx.input.entity
            ),
        ],
    )


def _get_import_schema_code(
    existing_code: str, app_name: str, entity_name: str
) -> str | None:
    snake_entity_name = to_snake_case(entity_name)
    pascal_entity_name = to_pascal_case(entity_name)
    import_module_path = f"{app_name}.schema.{snake_entity_name}"
    import_schema_code = f"from {import_module_path} import {pascal_entity_name}"
    if import_schema_code in existing_code:
        return None
    return import_schema_code


def _get_entity_metadata_assignment_code(
    existing_code: str, entity_name: str
) -> str | None:
    pascal_entity_name = to_pascal_case(entity_name)
    entity_metadata_assignment_code = "\n".join(
        [
            f"{pascal_entity_name}.metadata = metadata",
            f"{pascal_entity_name}.__table__.tometadata(metadata)",
        ]
    )
    if entity_metadata_assignment_code in existing_code:
        return None
    return entity_metadata_assignment_code


def update_any_client(ctx: AnyContext, any_client_file_path: str):
    existing_any_client_code = read_file(any_client_file_path)
    app_name = os.path.basename(APP_DIR)
    snake_entity_name = to_snake_case(ctx.input.entity)
    snake_plural_entity_name = to_snake_case(ctx.input.plural)
    pascal_entity_name = to_pascal_case(ctx.input.entity)
    any_client_method = read_file(
        file_path=os.path.join(
            os.path.dirname(__file__), "template", "any_client_method.py"
        ),
        replace_map={
            "my_entity": snake_entity_name,
            "my_entities": snake_plural_entity_name,
            "MyEntity": pascal_entity_name,
        },
    )
    new_code = append_code_to_class(
        existing_any_client_code, "AnyClient", any_client_method
    )
    write_file(
        file_path=any_client_file_path,
        content=[
            f"from {app_name}.schema.{snake_entity_name}.{snake_entity_name} import (",
            f"    {pascal_entity_name}CreateWithAudit, {pascal_entity_name}Response, {pascal_entity_name}UpdateWithAudit",  # noqa
            ")",
            new_code.strip(),
        ],
    )


def update_api_client(ctx: AnyContext, api_client_file_path: str):
    existing_api_client_code = read_file(api_client_file_path)
    upper_snake_module_name = to_snake_case(ctx.input.module).upper()
    app_name = os.path.basename(APP_DIR)
    snake_entity_name = to_snake_case(ctx.input.entity)
    snake_module_name = to_snake_case(ctx.input.module)
    write_file(
        file_path=api_client_file_path,
        content=[
            f"from {app_name}.module.{snake_module_name}.service.{snake_entity_name}.{snake_entity_name}_usecase import {snake_entity_name}_usecase",  # noqa
            prepend_code_to_module(
                prepend_parent_class(
                    existing_api_client_code, "APIClient", "user_api_client"
                ),
                f"user_api_client = user_usecase.as_api_client(base_url=APP_{upper_snake_module_name}_BASE_URL)",  # noqa
            ),
        ],
    )


def update_direct_client(ctx: AnyContext, direct_client_file_path: str):
    existing_direct_client_code = read_file(direct_client_file_path)
    app_name = os.path.basename(APP_DIR)
    snake_entity_name = to_snake_case(ctx.input.entity)
    snake_module_name = to_snake_case(ctx.input.module)
    write_file(
        file_path=direct_client_file_path,
        content=[
            f"from {app_name}.module.{snake_module_name}.service.{snake_entity_name}.{snake_entity_name}_usecase import {snake_entity_name}_usecase",  # noqa
            prepend_code_to_module(
                prepend_parent_class(
                    existing_direct_client_code, "DirectClient", "user_direct_client"
                ),
                "user_direct_client = user_usecase.as_direct_client()",
            ).strip(),
        ],
    )


def update_route(ctx: AnyContext, route_file_path: str):
    existing_route_code = read_file(route_file_path)
    entity_name = to_snake_case(ctx.input.entity)
    app_name = os.path.basename(APP_DIR)
    module_name = to_snake_case(ctx.input.module)
    write_file(
        file_path=route_file_path,
        content=[
            f"from {app_name}.module.{module_name}.service.{entity_name}.{entity_name}_usecase import {entity_name}_usecase",  # noqa
            append_code_to_function(
                existing_route_code,
                "serve_route",
                f"{entity_name}_usecase.serve_route(app)",
            ),
        ],
    )



from fastapi import FastAPI
from my_app_name.common.app import app
from my_app_name.common.schema import BasicResponse
from my_app_name.config import APP_MAIN_MODULE, APP_MODE, APP_MODULES
from my_app_name.module.auth.client.factory import client as auth_client
from my_app_name.schema.user import UserCreate, UserResponse


def serve_health_check(app: FastAPI):
    @app.api_route("/health", methods=["GET", "HEAD"], response_model=BasicResponse)
    async def health():
        """
        My App Name's health check
        """
        return BasicResponse(message="ok")


def serve_readiness_check(app: FastAPI):
    @app.api_route("/readiness", methods=["GET", "HEAD"], response_model=BasicResponse)
    async def readiness():
        """
        My App Name's readiness check
        """
        return BasicResponse(message="ok")


def serve_route(app: FastAPI):
    if APP_MODE != "monolith" and "gateway" not in APP_MODULES:
        return
    if APP_MODE == "monolith" or APP_MAIN_MODULE == "gateway":
        serve_health_check(app)
        serve_readiness_check(app)

    @app.get("/api/v1/users", response_model=list[UserResponse])
    async def auth_get_all_users() -> UserResponse:
        return await auth_client.get_all_users()

    @app.post("/api/v1/users", response_model=UserResponse | list[UserResponse])
    async def auth_create_user(data: UserCreate | list[UserCreate]):
        return await auth_client.create_user(data)


serve_route(app)



from my_app_name.module.auth.client.any_client import AnyClient
from my_app_name.module.auth.service.user.user_usecase import user_usecase


class DirectClient(user_usecase.as_direct_client(), AnyClient):
    pass
